# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: loki_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'loki_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12loki_service.proto\x12\x08isa.loki\x1a\x0c\x63ommon.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/protobuf/struct.proto\"\xb9\x01\n\x08LogEntry\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04line\x18\x02 \x01(\t\x12.\n\x06labels\x18\x03 \x03(\x0b\x32\x1e.isa.loki.LogEntry.LabelsEntry\x12\x11\n\tstream_id\x18\x04 \x01(\t\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xb5\x02\n\nStreamInfo\x12\x11\n\tstream_id\x18\x01 \x01(\t\x12\x30\n\x06labels\x18\x02 \x03(\x0b\x32 .isa.loki.StreamInfo.LabelsEntry\x12\x13\n\x0b\x65ntry_count\x18\x03 \x01(\x03\x12\r\n\x05\x62ytes\x18\x04 \x01(\x03\x12\x32\n\x0e\x66irst_entry_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x31\n\rlast_entry_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0f\n\x07user_id\x18\x07 \x01(\t\x12\x17\n\x0forganization_id\x18\x08 \x01(\t\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"I\n\tDataPoint\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05value\x18\x02 \x01(\x01\"\x8a\x01\n\x06Matrix\x12,\n\x06metric\x18\x01 \x03(\x0b\x32\x1c.isa.loki.Matrix.MetricEntry\x12#\n\x06values\x18\x02 \x03(\x0b\x32\x13.isa.loki.DataPoint\x1a-\n\x0bMetricEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"p\n\x0ePushLogRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\ttenant_id\x18\x03 \x01(\t\x12!\n\x05\x65ntry\x18\x04 \x01(\x0b\x32\x12.isa.loki.LogEntry\"F\n\x0fPushLogResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x11\n\tstream_id\x18\x03 \x01(\t\"w\n\x13PushLogBatchRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\ttenant_id\x18\x03 \x01(\t\x12#\n\x07\x65ntries\x18\x04 \x03(\x0b\x32\x12.isa.loki.LogEntry\"x\n\x14PushLogBatchResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x16\n\x0e\x61\x63\x63\x65pted_count\x18\x02 \x01(\x05\x12\x16\n\x0erejected_count\x18\x03 \x01(\x05\x12\x0e\n\x06\x65rrors\x18\x04 \x03(\t\x12\x0f\n\x07message\x18\x05 \x01(\t\"\xaf\x02\n\x14PushSimpleLogRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0f\n\x07service\x18\x03 \x01(\t\x12!\n\x05level\x18\x04 \x01(\x0e\x32\x12.isa.loki.LogLevel\x12\x0f\n\x07message\x18\x05 \x01(\t\x12\x45\n\x0c\x65xtra_labels\x18\x06 \x03(\x0b\x32/.isa.loki.PushSimpleLogRequest.ExtraLabelsEntry\x12-\n\ttimestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\x32\n\x10\x45xtraLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xd4\x01\n\x10QueryLogsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\ttenant_id\x18\x03 \x01(\t\x12\r\n\x05query\x18\x04 \x01(\t\x12\r\n\x05limit\x18\x05 \x01(\x05\x12)\n\x05start\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x11\n\tdirection\x18\x08 \x01(\t\"\xb2\x01\n\x11QueryLogsResponse\x12#\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x12.isa.loki.LogEntry\x12\x13\n\x0btotal_count\x18\x02 \x01(\x03\x12\x35\n\x05stats\x18\x03 \x03(\x0b\x32&.isa.loki.QueryLogsResponse.StatsEntry\x1a,\n\nStatsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xd0\x01\n\x11QueryRangeRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\ttenant_id\x18\x03 \x01(\t\x12\r\n\x05query\x18\x04 \x01(\t\x12)\n\x05start\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04step\x18\x07 \x01(\x05\x12\r\n\x05limit\x18\x08 \x01(\x05\"\xe9\x01\n\x12QueryRangeResponse\x12\x13\n\x0bresult_type\x18\x01 \x01(\t\x12(\n\x0ematrix_results\x18\x02 \x03(\x0b\x32\x10.isa.loki.Matrix\x12.\n\x0estream_results\x18\x03 \x03(\x0b\x32\x16.isa.loki.StreamResult\x12\x36\n\x05stats\x18\x04 \x03(\x0b\x32\'.isa.loki.QueryRangeResponse.StatsEntry\x1a,\n\nStatsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x96\x01\n\x0cStreamResult\x12\x32\n\x06stream\x18\x01 \x03(\x0b\x32\".isa.loki.StreamResult.StreamEntry\x12#\n\x07\x65ntries\x18\x02 \x03(\x0b\x32\x12.isa.loki.LogEntry\x1a-\n\x0bStreamEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xaa\x01\n\x0fTailLogsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\ttenant_id\x18\x03 \x01(\t\x12\r\n\x05query\x18\x04 \x01(\t\x12\x11\n\tdelay_for\x18\x05 \x01(\x05\x12\r\n\x05limit\x18\x06 \x01(\x05\x12)\n\x05start\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xb3\x01\n\x11QueryStatsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\ttenant_id\x18\x03 \x01(\t\x12\r\n\x05query\x18\x04 \x01(\t\x12)\n\x05start\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x8d\x02\n\x12QueryStatsResponse\x12\x15\n\rtotal_entries\x18\x01 \x01(\x03\x12\x13\n\x0btotal_bytes\x18\x02 \x01(\x03\x12\x15\n\rstreams_count\x18\x03 \x01(\x03\x12\x15\n\rquery_time_ms\x18\x04 \x01(\x01\x12O\n\x12level_distribution\x18\x05 \x03(\x0b\x32\x33.isa.loki.QueryStatsResponse.LevelDistributionEntry\x12\x12\n\ntop_labels\x18\x06 \x03(\t\x1a\x38\n\x16LevelDistributionEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\"\xa3\x01\n\x10GetLabelsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\ttenant_id\x18\x03 \x01(\t\x12)\n\x05start\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"#\n\x11GetLabelsResponse\x12\x0e\n\x06labels\x18\x01 \x03(\t\"\xcb\x01\n\x15GetLabelValuesRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\ttenant_id\x18\x03 \x01(\t\x12\x12\n\nlabel_name\x18\x04 \x01(\t\x12)\n\x05start\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05query\x18\x07 \x01(\t\"(\n\x16GetLabelValuesResponse\x12\x0e\n\x06values\x18\x01 \x03(\t\"\x83\x01\n\x15ValidateLabelsRequest\x12;\n\x06labels\x18\x01 \x03(\x0b\x32+.isa.loki.ValidateLabelsRequest.LabelsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"L\n\x16ValidateLabelsResponse\x12\r\n\x05valid\x18\x01 \x01(\x08\x12\x12\n\nviolations\x18\x02 \x03(\t\x12\x0f\n\x07message\x18\x03 \x01(\t\"\xeb\x01\n\x12ListStreamsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\ttenant_id\x18\x03 \x01(\t\x12\x43\n\x0clabel_filter\x18\x04 \x03(\x0b\x32-.isa.loki.ListStreamsRequest.LabelFilterEntry\x12\x0c\n\x04page\x18\x05 \x01(\x05\x12\x11\n\tpage_size\x18\x06 \x01(\x05\x1a\x32\n\x10LabelFilterEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"r\n\x13ListStreamsResponse\x12%\n\x07streams\x18\x01 \x03(\x0b\x32\x14.isa.loki.StreamInfo\x12\x13\n\x0btotal_count\x18\x02 \x01(\x05\x12\x0c\n\x04page\x18\x03 \x01(\x05\x12\x11\n\tpage_size\x18\x04 \x01(\x05\":\n\x14GetStreamInfoRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x11\n\tstream_id\x18\x02 \x01(\t\"=\n\x15GetStreamInfoResponse\x12$\n\x06stream\x18\x01 \x01(\x0b\x32\x14.isa.loki.StreamInfo\"\x8d\x01\n\x13\x44\x65leteStreamRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x11\n\tstream_id\x18\x02 \x01(\t\x12)\n\x05start\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"Q\n\x14\x44\x65leteStreamResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x17\n\x0f\x64\x65leted_entries\x18\x02 \x01(\x03\x12\x0f\n\x07message\x18\x03 \x01(\t\"\xef\x01\n\x11\x45xportLogsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\ttenant_id\x18\x03 \x01(\t\x12\r\n\x05query\x18\x04 \x01(\t\x12)\n\x05start\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12&\n\x06\x66ormat\x18\x07 \x01(\x0e\x32\x16.isa.loki.ExportFormat\x12\x12\n\nchunk_size\x18\x08 \x01(\x05\"y\n\x12\x45xportLogsResponse\x12\x11\n\texport_id\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x15\n\rtotal_entries\x18\x03 \x01(\x03\x12\x19\n\x11processed_entries\x18\x04 \x01(\x03\x12\x10\n\x08\x63omplete\x18\x05 \x01(\x08\"<\n\x16GetExportStatusRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x11\n\texport_id\x18\x02 \x01(\t\"\x83\x02\n\x17GetExportStatusResponse\x12\x11\n\texport_id\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\x15\n\rtotal_entries\x18\x03 \x01(\x03\x12\x18\n\x10\x65xported_entries\x18\x04 \x01(\x03\x12\x1b\n\x13progress_percentage\x18\x05 \x01(\x05\x12.\n\ncreated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0c\x63ompleted_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\rerror_message\x18\x08 \x01(\t\"\x94\x01\n\x14GetStatisticsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12)\n\x05start\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xa1\x04\n\x15GetStatisticsResponse\x12\x15\n\rtotal_entries\x18\x01 \x01(\x03\x12\x13\n\x0btotal_bytes\x18\x02 \x01(\x03\x12\x15\n\rstreams_count\x18\x03 \x01(\x03\x12Q\n\x12\x65ntries_by_service\x18\x04 \x03(\x0b\x32\x35.isa.loki.GetStatisticsResponse.EntriesByServiceEntry\x12M\n\x10\x65ntries_by_level\x18\x05 \x03(\x0b\x32\x33.isa.loki.GetStatisticsResponse.EntriesByLevelEntry\x12+\n\x0eingestion_rate\x18\x06 \x03(\x0b\x32\x13.isa.loki.DataPoint\x12\x14\n\x0ctop_services\x18\x07 \x03(\t\x12?\n\x08metadata\x18\x08 \x03(\x0b\x32-.isa.loki.GetStatisticsResponse.MetadataEntry\x1a\x37\n\x15\x45ntriesByServiceEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\x1a\x35\n\x13\x45ntriesByLevelEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"?\n\x13GetUserQuotaRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\"\xa8\x01\n\x14GetUserQuotaResponse\x12\x13\n\x0b\x64\x61ily_limit\x18\x01 \x01(\x03\x12\x12\n\ntoday_used\x18\x02 \x01(\x03\x12\x1b\n\x13storage_limit_bytes\x18\x03 \x01(\x03\x12\x1a\n\x12storage_used_bytes\x18\x04 \x01(\x03\x12\x16\n\x0eretention_days\x18\x05 \x01(\x05\x12\x16\n\x0equota_exceeded\x18\x06 \x01(\x08\",\n\x16LokiHealthCheckRequest\x12\x12\n\ndeep_check\x18\x01 \x01(\x08\"\xaf\x02\n\x17LokiHealthCheckResponse\x12\x0f\n\x07healthy\x18\x01 \x01(\x08\x12\x13\n\x0bloki_status\x18\x02 \x01(\t\x12\x11\n\tcan_write\x18\x03 \x01(\x08\x12\x10\n\x08\x63\x61n_read\x18\x04 \x01(\x08\x12.\n\nchecked_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0f\n\x07message\x18\x06 \x01(\t\x12P\n\x10\x63omponent_status\x18\x07 \x03(\x0b\x32\x36.isa.loki.LokiHealthCheckResponse.ComponentStatusEntry\x1a\x36\n\x14\x43omponentStatusEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01*\x88\x01\n\x08LogLevel\x12\x15\n\x11LOG_LEVEL_UNKNOWN\x10\x00\x12\x13\n\x0fLOG_LEVEL_DEBUG\x10\x01\x12\x12\n\x0eLOG_LEVEL_INFO\x10\x02\x12\x12\n\x0eLOG_LEVEL_WARN\x10\x03\x12\x13\n\x0fLOG_LEVEL_ERROR\x10\x04\x12\x13\n\x0fLOG_LEVEL_FATAL\x10\x05*R\n\x0c\x45xportFormat\x12\x0f\n\x0b\x45XPORT_JSON\x10\x00\x12\x10\n\x0c\x45XPORT_JSONL\x10\x01\x12\x0e\n\nEXPORT_CSV\x10\x02\x12\x0f\n\x0b\x45XPORT_TEXT\x10\x03\x32\xbf\x0b\n\x0bLokiService\x12>\n\x07PushLog\x12\x18.isa.loki.PushLogRequest\x1a\x19.isa.loki.PushLogResponse\x12M\n\x0cPushLogBatch\x12\x1d.isa.loki.PushLogBatchRequest\x1a\x1e.isa.loki.PushLogBatchResponse\x12K\n\rPushLogStream\x12\x18.isa.loki.PushLogRequest\x1a\x1e.isa.loki.PushLogBatchResponse(\x01\x12J\n\rPushSimpleLog\x12\x1e.isa.loki.PushSimpleLogRequest\x1a\x19.isa.loki.PushLogResponse\x12\x44\n\tQueryLogs\x12\x1a.isa.loki.QueryLogsRequest\x1a\x1b.isa.loki.QueryLogsResponse\x12G\n\nQueryRange\x12\x1b.isa.loki.QueryRangeRequest\x1a\x1c.isa.loki.QueryRangeResponse\x12;\n\x08TailLogs\x12\x19.isa.loki.TailLogsRequest\x1a\x12.isa.loki.LogEntry0\x01\x12G\n\nQueryStats\x12\x1b.isa.loki.QueryStatsRequest\x1a\x1c.isa.loki.QueryStatsResponse\x12\x44\n\tGetLabels\x12\x1a.isa.loki.GetLabelsRequest\x1a\x1b.isa.loki.GetLabelsResponse\x12S\n\x0eGetLabelValues\x12\x1f.isa.loki.GetLabelValuesRequest\x1a .isa.loki.GetLabelValuesResponse\x12S\n\x0eValidateLabels\x12\x1f.isa.loki.ValidateLabelsRequest\x1a .isa.loki.ValidateLabelsResponse\x12J\n\x0bListStreams\x12\x1c.isa.loki.ListStreamsRequest\x1a\x1d.isa.loki.ListStreamsResponse\x12P\n\rGetStreamInfo\x12\x1e.isa.loki.GetStreamInfoRequest\x1a\x1f.isa.loki.GetStreamInfoResponse\x12M\n\x0c\x44\x65leteStream\x12\x1d.isa.loki.DeleteStreamRequest\x1a\x1e.isa.loki.DeleteStreamResponse\x12I\n\nExportLogs\x12\x1b.isa.loki.ExportLogsRequest\x1a\x1c.isa.loki.ExportLogsResponse0\x01\x12V\n\x0fGetExportStatus\x12 .isa.loki.GetExportStatusRequest\x1a!.isa.loki.GetExportStatusResponse\x12P\n\rGetStatistics\x12\x1e.isa.loki.GetStatisticsRequest\x1a\x1f.isa.loki.GetStatisticsResponse\x12M\n\x0cGetUserQuota\x12\x1d.isa.loki.GetUserQuotaRequest\x1a\x1e.isa.loki.GetUserQuotaResponse\x12R\n\x0bHealthCheck\x12 .isa.loki.LokiHealthCheckRequest\x1a!.isa.loki.LokiHealthCheckResponseB/Z-github.com/isa-cloud/isa_cloud/api/proto/lokib\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'loki_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z-github.com/isa-cloud/isa_cloud/api/proto/loki'
  _globals['_LOGENTRY_LABELSENTRY']._loaded_options = None
  _globals['_LOGENTRY_LABELSENTRY']._serialized_options = b'8\001'
  _globals['_STREAMINFO_LABELSENTRY']._loaded_options = None
  _globals['_STREAMINFO_LABELSENTRY']._serialized_options = b'8\001'
  _globals['_MATRIX_METRICENTRY']._loaded_options = None
  _globals['_MATRIX_METRICENTRY']._serialized_options = b'8\001'
  _globals['_PUSHSIMPLELOGREQUEST_EXTRALABELSENTRY']._loaded_options = None
  _globals['_PUSHSIMPLELOGREQUEST_EXTRALABELSENTRY']._serialized_options = b'8\001'
  _globals['_QUERYLOGSRESPONSE_STATSENTRY']._loaded_options = None
  _globals['_QUERYLOGSRESPONSE_STATSENTRY']._serialized_options = b'8\001'
  _globals['_QUERYRANGERESPONSE_STATSENTRY']._loaded_options = None
  _globals['_QUERYRANGERESPONSE_STATSENTRY']._serialized_options = b'8\001'
  _globals['_STREAMRESULT_STREAMENTRY']._loaded_options = None
  _globals['_STREAMRESULT_STREAMENTRY']._serialized_options = b'8\001'
  _globals['_QUERYSTATSRESPONSE_LEVELDISTRIBUTIONENTRY']._loaded_options = None
  _globals['_QUERYSTATSRESPONSE_LEVELDISTRIBUTIONENTRY']._serialized_options = b'8\001'
  _globals['_VALIDATELABELSREQUEST_LABELSENTRY']._loaded_options = None
  _globals['_VALIDATELABELSREQUEST_LABELSENTRY']._serialized_options = b'8\001'
  _globals['_LISTSTREAMSREQUEST_LABELFILTERENTRY']._loaded_options = None
  _globals['_LISTSTREAMSREQUEST_LABELFILTERENTRY']._serialized_options = b'8\001'
  _globals['_GETSTATISTICSRESPONSE_ENTRIESBYSERVICEENTRY']._loaded_options = None
  _globals['_GETSTATISTICSRESPONSE_ENTRIESBYSERVICEENTRY']._serialized_options = b'8\001'
  _globals['_GETSTATISTICSRESPONSE_ENTRIESBYLEVELENTRY']._loaded_options = None
  _globals['_GETSTATISTICSRESPONSE_ENTRIESBYLEVELENTRY']._serialized_options = b'8\001'
  _globals['_GETSTATISTICSRESPONSE_METADATAENTRY']._loaded_options = None
  _globals['_GETSTATISTICSRESPONSE_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_LOKIHEALTHCHECKRESPONSE_COMPONENTSTATUSENTRY']._loaded_options = None
  _globals['_LOKIHEALTHCHECKRESPONSE_COMPONENTSTATUSENTRY']._serialized_options = b'8\001'
  _globals['_LOGLEVEL']._serialized_start=6527
  _globals['_LOGLEVEL']._serialized_end=6663
  _globals['_EXPORTFORMAT']._serialized_start=6665
  _globals['_EXPORTFORMAT']._serialized_end=6747
  _globals['_LOGENTRY']._serialized_start=110
  _globals['_LOGENTRY']._serialized_end=295
  _globals['_LOGENTRY_LABELSENTRY']._serialized_start=250
  _globals['_LOGENTRY_LABELSENTRY']._serialized_end=295
  _globals['_STREAMINFO']._serialized_start=298
  _globals['_STREAMINFO']._serialized_end=607
  _globals['_STREAMINFO_LABELSENTRY']._serialized_start=250
  _globals['_STREAMINFO_LABELSENTRY']._serialized_end=295
  _globals['_DATAPOINT']._serialized_start=609
  _globals['_DATAPOINT']._serialized_end=682
  _globals['_MATRIX']._serialized_start=685
  _globals['_MATRIX']._serialized_end=823
  _globals['_MATRIX_METRICENTRY']._serialized_start=778
  _globals['_MATRIX_METRICENTRY']._serialized_end=823
  _globals['_PUSHLOGREQUEST']._serialized_start=825
  _globals['_PUSHLOGREQUEST']._serialized_end=937
  _globals['_PUSHLOGRESPONSE']._serialized_start=939
  _globals['_PUSHLOGRESPONSE']._serialized_end=1009
  _globals['_PUSHLOGBATCHREQUEST']._serialized_start=1011
  _globals['_PUSHLOGBATCHREQUEST']._serialized_end=1130
  _globals['_PUSHLOGBATCHRESPONSE']._serialized_start=1132
  _globals['_PUSHLOGBATCHRESPONSE']._serialized_end=1252
  _globals['_PUSHSIMPLELOGREQUEST']._serialized_start=1255
  _globals['_PUSHSIMPLELOGREQUEST']._serialized_end=1558
  _globals['_PUSHSIMPLELOGREQUEST_EXTRALABELSENTRY']._serialized_start=1508
  _globals['_PUSHSIMPLELOGREQUEST_EXTRALABELSENTRY']._serialized_end=1558
  _globals['_QUERYLOGSREQUEST']._serialized_start=1561
  _globals['_QUERYLOGSREQUEST']._serialized_end=1773
  _globals['_QUERYLOGSRESPONSE']._serialized_start=1776
  _globals['_QUERYLOGSRESPONSE']._serialized_end=1954
  _globals['_QUERYLOGSRESPONSE_STATSENTRY']._serialized_start=1910
  _globals['_QUERYLOGSRESPONSE_STATSENTRY']._serialized_end=1954
  _globals['_QUERYRANGEREQUEST']._serialized_start=1957
  _globals['_QUERYRANGEREQUEST']._serialized_end=2165
  _globals['_QUERYRANGERESPONSE']._serialized_start=2168
  _globals['_QUERYRANGERESPONSE']._serialized_end=2401
  _globals['_QUERYRANGERESPONSE_STATSENTRY']._serialized_start=1910
  _globals['_QUERYRANGERESPONSE_STATSENTRY']._serialized_end=1954
  _globals['_STREAMRESULT']._serialized_start=2404
  _globals['_STREAMRESULT']._serialized_end=2554
  _globals['_STREAMRESULT_STREAMENTRY']._serialized_start=2509
  _globals['_STREAMRESULT_STREAMENTRY']._serialized_end=2554
  _globals['_TAILLOGSREQUEST']._serialized_start=2557
  _globals['_TAILLOGSREQUEST']._serialized_end=2727
  _globals['_QUERYSTATSREQUEST']._serialized_start=2730
  _globals['_QUERYSTATSREQUEST']._serialized_end=2909
  _globals['_QUERYSTATSRESPONSE']._serialized_start=2912
  _globals['_QUERYSTATSRESPONSE']._serialized_end=3181
  _globals['_QUERYSTATSRESPONSE_LEVELDISTRIBUTIONENTRY']._serialized_start=3125
  _globals['_QUERYSTATSRESPONSE_LEVELDISTRIBUTIONENTRY']._serialized_end=3181
  _globals['_GETLABELSREQUEST']._serialized_start=3184
  _globals['_GETLABELSREQUEST']._serialized_end=3347
  _globals['_GETLABELSRESPONSE']._serialized_start=3349
  _globals['_GETLABELSRESPONSE']._serialized_end=3384
  _globals['_GETLABELVALUESREQUEST']._serialized_start=3387
  _globals['_GETLABELVALUESREQUEST']._serialized_end=3590
  _globals['_GETLABELVALUESRESPONSE']._serialized_start=3592
  _globals['_GETLABELVALUESRESPONSE']._serialized_end=3632
  _globals['_VALIDATELABELSREQUEST']._serialized_start=3635
  _globals['_VALIDATELABELSREQUEST']._serialized_end=3766
  _globals['_VALIDATELABELSREQUEST_LABELSENTRY']._serialized_start=250
  _globals['_VALIDATELABELSREQUEST_LABELSENTRY']._serialized_end=295
  _globals['_VALIDATELABELSRESPONSE']._serialized_start=3768
  _globals['_VALIDATELABELSRESPONSE']._serialized_end=3844
  _globals['_LISTSTREAMSREQUEST']._serialized_start=3847
  _globals['_LISTSTREAMSREQUEST']._serialized_end=4082
  _globals['_LISTSTREAMSREQUEST_LABELFILTERENTRY']._serialized_start=4032
  _globals['_LISTSTREAMSREQUEST_LABELFILTERENTRY']._serialized_end=4082
  _globals['_LISTSTREAMSRESPONSE']._serialized_start=4084
  _globals['_LISTSTREAMSRESPONSE']._serialized_end=4198
  _globals['_GETSTREAMINFOREQUEST']._serialized_start=4200
  _globals['_GETSTREAMINFOREQUEST']._serialized_end=4258
  _globals['_GETSTREAMINFORESPONSE']._serialized_start=4260
  _globals['_GETSTREAMINFORESPONSE']._serialized_end=4321
  _globals['_DELETESTREAMREQUEST']._serialized_start=4324
  _globals['_DELETESTREAMREQUEST']._serialized_end=4465
  _globals['_DELETESTREAMRESPONSE']._serialized_start=4467
  _globals['_DELETESTREAMRESPONSE']._serialized_end=4548
  _globals['_EXPORTLOGSREQUEST']._serialized_start=4551
  _globals['_EXPORTLOGSREQUEST']._serialized_end=4790
  _globals['_EXPORTLOGSRESPONSE']._serialized_start=4792
  _globals['_EXPORTLOGSRESPONSE']._serialized_end=4913
  _globals['_GETEXPORTSTATUSREQUEST']._serialized_start=4915
  _globals['_GETEXPORTSTATUSREQUEST']._serialized_end=4975
  _globals['_GETEXPORTSTATUSRESPONSE']._serialized_start=4978
  _globals['_GETEXPORTSTATUSRESPONSE']._serialized_end=5237
  _globals['_GETSTATISTICSREQUEST']._serialized_start=5240
  _globals['_GETSTATISTICSREQUEST']._serialized_end=5388
  _globals['_GETSTATISTICSRESPONSE']._serialized_start=5391
  _globals['_GETSTATISTICSRESPONSE']._serialized_end=5936
  _globals['_GETSTATISTICSRESPONSE_ENTRIESBYSERVICEENTRY']._serialized_start=5777
  _globals['_GETSTATISTICSRESPONSE_ENTRIESBYSERVICEENTRY']._serialized_end=5832
  _globals['_GETSTATISTICSRESPONSE_ENTRIESBYLEVELENTRY']._serialized_start=5834
  _globals['_GETSTATISTICSRESPONSE_ENTRIESBYLEVELENTRY']._serialized_end=5887
  _globals['_GETSTATISTICSRESPONSE_METADATAENTRY']._serialized_start=5889
  _globals['_GETSTATISTICSRESPONSE_METADATAENTRY']._serialized_end=5936
  _globals['_GETUSERQUOTAREQUEST']._serialized_start=5938
  _globals['_GETUSERQUOTAREQUEST']._serialized_end=6001
  _globals['_GETUSERQUOTARESPONSE']._serialized_start=6004
  _globals['_GETUSERQUOTARESPONSE']._serialized_end=6172
  _globals['_LOKIHEALTHCHECKREQUEST']._serialized_start=6174
  _globals['_LOKIHEALTHCHECKREQUEST']._serialized_end=6218
  _globals['_LOKIHEALTHCHECKRESPONSE']._serialized_start=6221
  _globals['_LOKIHEALTHCHECKRESPONSE']._serialized_end=6524
  _globals['_LOKIHEALTHCHECKRESPONSE_COMPONENTSTATUSENTRY']._serialized_start=6470
  _globals['_LOKIHEALTHCHECKRESPONSE_COMPONENTSTATUSENTRY']._serialized_end=6524
  _globals['_LOKISERVICE']._serialized_start=6750
  _globals['_LOKISERVICE']._serialized_end=8221
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: postgres_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'postgres_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16postgres_service.proto\x12\x0cisa.postgres\x1a\x0c\x63ommon.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/protobuf/struct.proto\"\x82\x01\n\x0cQueryRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x0b\n\x03sql\x18\x02 \x01(\t\x12&\n\x06params\x18\x03 \x03(\x0b\x32\x16.google.protobuf.Value\x12\x0e\n\x06schema\x18\x04 \x01(\t\"y\n\rQueryResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12%\n\x04rows\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x11\n\trow_count\x18\x03 \x01(\x05\"\x85\x01\n\x0fQueryRowRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x0b\n\x03sql\x18\x02 \x01(\t\x12&\n\x06params\x18\x03 \x03(\x0b\x32\x16.google.protobuf.Value\x12\x0e\n\x06schema\x18\x04 \x01(\t\"w\n\x10QueryRowResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12$\n\x03row\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\r\n\x05\x66ound\x18\x03 \x01(\x08\"\x84\x01\n\x0e\x45xecuteRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x0b\n\x03sql\x18\x02 \x01(\t\x12&\n\x06params\x18\x03 \x03(\x0b\x32\x16.google.protobuf.Value\x12\x0e\n\x06schema\x18\x04 \x01(\t\"m\n\x0f\x45xecuteResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x15\n\rrows_affected\x18\x02 \x01(\x03\x12\x13\n\x0b\x63ommand_tag\x18\x03 \x01(\t\"\x86\x01\n\x13\x45xecuteBatchRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x30\n\noperations\x18\x02 \x03(\x0b\x32\x1c.isa.postgres.BatchOperation\x12\x0e\n\x06schema\x18\x03 \x01(\t\"E\n\x0e\x42\x61tchOperation\x12\x0b\n\x03sql\x18\x01 \x01(\t\x12&\n\x06params\x18\x02 \x03(\x0b\x32\x16.google.protobuf.Value\"\x91\x01\n\x14\x45xecuteBatchResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12,\n\x07results\x18\x02 \x03(\x0b\x32\x1b.isa.postgres.ExecuteResult\x12\x1b\n\x13total_rows_affected\x18\x03 \x01(\x05\"J\n\rExecuteResult\x12\x15\n\rrows_affected\x18\x01 \x01(\x03\x12\x13\n\x0b\x63ommand_tag\x18\x02 \x01(\t\x12\r\n\x05\x65rror\x18\x03 \x01(\t\"\xcd\x01\n\x11SelectFromRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\r\n\x05table\x18\x02 \x01(\t\x12\x0f\n\x07\x63olumns\x18\x03 \x03(\t\x12(\n\x05where\x18\x04 \x03(\x0b\x32\x19.isa.postgres.WhereClause\x12\x10\n\x08order_by\x18\x05 \x03(\t\x12\r\n\x05limit\x18\x06 \x01(\x05\x12\x0e\n\x06offset\x18\x07 \x01(\x05\x12\x0e\n\x06schema\x18\x08 \x01(\t\"V\n\x0bWhereClause\x12\x0e\n\x06\x63olumn\x18\x01 \x01(\t\x12\x10\n\x08operator\x18\x02 \x01(\t\x12%\n\x05value\x18\x03 \x01(\x0b\x32\x16.google.protobuf.Value\"~\n\x12SelectFromResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12%\n\x04rows\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x11\n\trow_count\x18\x03 \x01(\x05\"\x9b\x01\n\x11InsertIntoRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\r\n\x05table\x18\x02 \x01(\t\x12%\n\x04rows\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x11\n\treturning\x18\x04 \x01(\x08\x12\x0e\n\x06schema\x18\x05 \x01(\t\"\x82\x01\n\x12InsertIntoResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x15\n\rrows_inserted\x18\x02 \x01(\x03\x12%\n\x04rows\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct\"\xc8\x01\n\x12UpdateTableRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\r\n\x05table\x18\x02 \x01(\t\x12\'\n\x06values\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\x12(\n\x05where\x18\x04 \x03(\x0b\x32\x19.isa.postgres.WhereClause\x12\x11\n\treturning\x18\x05 \x01(\x08\x12\x0e\n\x06schema\x18\x06 \x01(\t\"\x82\x01\n\x13UpdateTableResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x14\n\x0crows_updated\x18\x02 \x01(\x03\x12%\n\x04rows\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct\"\x9e\x01\n\x11\x44\x65leteFromRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\r\n\x05table\x18\x02 \x01(\t\x12(\n\x05where\x18\x03 \x03(\x0b\x32\x19.isa.postgres.WhereClause\x12\x11\n\treturning\x18\x04 \x01(\x08\x12\x0e\n\x06schema\x18\x05 \x01(\t\"\x81\x01\n\x12\x44\x65leteFromResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x14\n\x0crows_deleted\x18\x02 \x01(\x03\x12%\n\x04rows\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct\"a\n\x17\x42\x65ginTransactionRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x17\n\x0fisolation_level\x18\x02 \x01(\t\"b\n\x18\x42\x65ginTransactionResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x16\n\x0etransaction_id\x18\x02 \x01(\t\"a\n\x18\x43ommitTransactionRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x16\n\x0etransaction_id\x18\x02 \x01(\t\"\\\n\x19\x43ommitTransactionResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0f\n\x07success\x18\x02 \x01(\x08\"c\n\x1aRollbackTransactionRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x16\n\x0etransaction_id\x18\x02 \x01(\t\"^\n\x1bRollbackTransactionResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0f\n\x07success\x18\x02 \x01(\x08\"\x8e\x01\n\x1b\x45xecuteInTransactionRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x30\n\noperations\x18\x02 \x03(\x0b\x32\x1c.isa.postgres.BatchOperation\x12\x0e\n\x06schema\x18\x03 \x01(\t\"\x8f\x01\n\x1c\x45xecuteInTransactionResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12,\n\x07results\x18\x02 \x03(\x0b\x32\x1b.isa.postgres.ExecuteResult\x12\x11\n\tcommitted\x18\x03 \x01(\x08\"c\n\x13GetTableInfoRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\r\n\x05table\x18\x02 \x01(\t\x12\x0e\n\x06schema\x18\x03 \x01(\t\"s\n\x14GetTableInfoResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12+\n\ntable_info\x18\x02 \x01(\x0b\x32\x17.isa.postgres.TableInfo\"\x8c\x01\n\tTableInfo\x12\x12\n\ntable_name\x18\x01 \x01(\t\x12\x0e\n\x06schema\x18\x02 \x01(\t\x12)\n\x07\x63olumns\x18\x03 \x03(\x0b\x32\x18.isa.postgres.ColumnInfo\x12\x14\n\x0cprimary_keys\x18\x04 \x03(\t\x12\x1a\n\x12row_count_estimate\x18\x05 \x01(\x03\"`\n\nColumnInfo\x12\x13\n\x0b\x63olumn_name\x18\x01 \x01(\t\x12\x11\n\tdata_type\x18\x02 \x01(\t\x12\x13\n\x0bis_nullable\x18\x03 \x01(\x08\x12\x15\n\rdefault_value\x18\x04 \x01(\t\"R\n\x11ListTablesRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x0e\n\x06schema\x18\x02 \x01(\t\"T\n\x12ListTablesResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0e\n\x06tables\x18\x02 \x03(\t\"b\n\x12TableExistsRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\r\n\x05table\x18\x02 \x01(\t\x12\x0e\n\x06schema\x18\x03 \x01(\t\"U\n\x13TableExistsResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0e\n\x06\x65xists\x18\x02 \x01(\x08\"U\n\x12HealthCheckRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\x12\x10\n\x08\x64\x65tailed\x18\x02 \x01(\x08\"\xa1\x01\n\x13HealthCheckResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x0f\n\x07healthy\x18\x02 \x01(\x08\x12\x0e\n\x06status\x18\x03 \x01(\t\x12\x0f\n\x07version\x18\x04 \x01(\t\x12(\n\x07\x64\x65tails\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\"@\n\x0fGetStatsRequest\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x1b.isa.common.RequestMetadata\"\xa8\x01\n\x10GetStatsResponse\x12.\n\x08metadata\x18\x01 \x01(\x0b\x32\x1c.isa.common.ResponseMetadata\x12\x35\n\npool_stats\x18\x02 \x01(\x0b\x32!.isa.postgres.ConnectionPoolStats\x12-\n\x08\x64\x62_stats\x18\x03 \x01(\x0b\x32\x1b.isa.postgres.DatabaseStats\"\x95\x01\n\x13\x43onnectionPoolStats\x12\x17\n\x0fmax_connections\x18\x01 \x01(\x05\x12\x18\n\x10open_connections\x18\x02 \x01(\x05\x12\x18\n\x10idle_connections\x18\x03 \x01(\x05\x12\x1a\n\x12\x61\x63tive_connections\x18\x04 \x01(\x05\x12\x15\n\rtotal_queries\x18\x05 \x01(\x03\"\xa0\x01\n\rDatabaseStats\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x1b\n\x13\x64\x61tabase_size_bytes\x18\x02 \x01(\x03\x12\x14\n\x0ctotal_tables\x18\x03 \x01(\x05\x12\x19\n\x11total_connections\x18\x04 \x01(\x03\x12\x30\n\x0cuptime_since\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp2\xc3\x0b\n\x0fPostgresService\x12@\n\x05Query\x12\x1a.isa.postgres.QueryRequest\x1a\x1b.isa.postgres.QueryResponse\x12I\n\x08QueryRow\x12\x1d.isa.postgres.QueryRowRequest\x1a\x1e.isa.postgres.QueryRowResponse\x12\x46\n\x07\x45xecute\x12\x1c.isa.postgres.ExecuteRequest\x1a\x1d.isa.postgres.ExecuteResponse\x12U\n\x0c\x45xecuteBatch\x12!.isa.postgres.ExecuteBatchRequest\x1a\".isa.postgres.ExecuteBatchResponse\x12O\n\nSelectFrom\x12\x1f.isa.postgres.SelectFromRequest\x1a .isa.postgres.SelectFromResponse\x12O\n\nInsertInto\x12\x1f.isa.postgres.InsertIntoRequest\x1a .isa.postgres.InsertIntoResponse\x12R\n\x0bUpdateTable\x12 .isa.postgres.UpdateTableRequest\x1a!.isa.postgres.UpdateTableResponse\x12O\n\nDeleteFrom\x12\x1f.isa.postgres.DeleteFromRequest\x1a .isa.postgres.DeleteFromResponse\x12\x61\n\x10\x42\x65ginTransaction\x12%.isa.postgres.BeginTransactionRequest\x1a&.isa.postgres.BeginTransactionResponse\x12\x64\n\x11\x43ommitTransaction\x12&.isa.postgres.CommitTransactionRequest\x1a\'.isa.postgres.CommitTransactionResponse\x12j\n\x13RollbackTransaction\x12(.isa.postgres.RollbackTransactionRequest\x1a).isa.postgres.RollbackTransactionResponse\x12m\n\x14\x45xecuteInTransaction\x12).isa.postgres.ExecuteInTransactionRequest\x1a*.isa.postgres.ExecuteInTransactionResponse\x12U\n\x0cGetTableInfo\x12!.isa.postgres.GetTableInfoRequest\x1a\".isa.postgres.GetTableInfoResponse\x12O\n\nListTables\x12\x1f.isa.postgres.ListTablesRequest\x1a .isa.postgres.ListTablesResponse\x12R\n\x0bTableExists\x12 .isa.postgres.TableExistsRequest\x1a!.isa.postgres.TableExistsResponse\x12R\n\x0bHealthCheck\x12 .isa.postgres.HealthCheckRequest\x1a!.isa.postgres.HealthCheckResponse\x12I\n\x08GetStats\x12\x1d.isa.postgres.GetStatsRequest\x1a\x1e.isa.postgres.GetStatsResponseB3Z1github.com/isa-cloud/isa_cloud/api/proto/postgresb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'postgres_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z1github.com/isa-cloud/isa_cloud/api/proto/postgres'
  _globals['_QUERYREQUEST']._serialized_start=118
  _globals['_QUERYREQUEST']._serialized_end=248
  _globals['_QUERYRESPONSE']._serialized_start=250
  _globals['_QUERYRESPONSE']._serialized_end=371
  _globals['_QUERYROWREQUEST']._serialized_start=374
  _globals['_QUERYROWREQUEST']._serialized_end=507
  _globals['_QUERYROWRESPONSE']._serialized_start=509
  _globals['_QUERYROWRESPONSE']._serialized_end=628
  _globals['_EXECUTEREQUEST']._serialized_start=631
  _globals['_EXECUTEREQUEST']._serialized_end=763
  _globals['_EXECUTERESPONSE']._serialized_start=765
  _globals['_EXECUTERESPONSE']._serialized_end=874
  _globals['_EXECUTEBATCHREQUEST']._serialized_start=877
  _globals['_EXECUTEBATCHREQUEST']._serialized_end=1011
  _globals['_BATCHOPERATION']._serialized_start=1013
  _globals['_BATCHOPERATION']._serialized_end=1082
  _globals['_EXECUTEBATCHRESPONSE']._serialized_start=1085
  _globals['_EXECUTEBATCHRESPONSE']._serialized_end=1230
  _globals['_EXECUTERESULT']._serialized_start=1232
  _globals['_EXECUTERESULT']._serialized_end=1306
  _globals['_SELECTFROMREQUEST']._serialized_start=1309
  _globals['_SELECTFROMREQUEST']._serialized_end=1514
  _globals['_WHERECLAUSE']._serialized_start=1516
  _globals['_WHERECLAUSE']._serialized_end=1602
  _globals['_SELECTFROMRESPONSE']._serialized_start=1604
  _globals['_SELECTFROMRESPONSE']._serialized_end=1730
  _globals['_INSERTINTOREQUEST']._serialized_start=1733
  _globals['_INSERTINTOREQUEST']._serialized_end=1888
  _globals['_INSERTINTORESPONSE']._serialized_start=1891
  _globals['_INSERTINTORESPONSE']._serialized_end=2021
  _globals['_UPDATETABLEREQUEST']._serialized_start=2024
  _globals['_UPDATETABLEREQUEST']._serialized_end=2224
  _globals['_UPDATETABLERESPONSE']._serialized_start=2227
  _globals['_UPDATETABLERESPONSE']._serialized_end=2357
  _globals['_DELETEFROMREQUEST']._serialized_start=2360
  _globals['_DELETEFROMREQUEST']._serialized_end=2518
  _globals['_DELETEFROMRESPONSE']._serialized_start=2521
  _globals['_DELETEFROMRESPONSE']._serialized_end=2650
  _globals['_BEGINTRANSACTIONREQUEST']._serialized_start=2652
  _globals['_BEGINTRANSACTIONREQUEST']._serialized_end=2749
  _globals['_BEGINTRANSACTIONRESPONSE']._serialized_start=2751
  _globals['_BEGINTRANSACTIONRESPONSE']._serialized_end=2849
  _globals['_COMMITTRANSACTIONREQUEST']._serialized_start=2851
  _globals['_COMMITTRANSACTIONREQUEST']._serialized_end=2948
  _globals['_COMMITTRANSACTIONRESPONSE']._serialized_start=2950
  _globals['_COMMITTRANSACTIONRESPONSE']._serialized_end=3042
  _globals['_ROLLBACKTRANSACTIONREQUEST']._serialized_start=3044
  _globals['_ROLLBACKTRANSACTIONREQUEST']._serialized_end=3143
  _globals['_ROLLBACKTRANSACTIONRESPONSE']._serialized_start=3145
  _globals['_ROLLBACKTRANSACTIONRESPONSE']._serialized_end=3239
  _globals['_EXECUTEINTRANSACTIONREQUEST']._serialized_start=3242
  _globals['_EXECUTEINTRANSACTIONREQUEST']._serialized_end=3384
  _globals['_EXECUTEINTRANSACTIONRESPONSE']._serialized_start=3387
  _globals['_EXECUTEINTRANSACTIONRESPONSE']._serialized_end=3530
  _globals['_GETTABLEINFOREQUEST']._serialized_start=3532
  _globals['_GETTABLEINFOREQUEST']._serialized_end=3631
  _globals['_GETTABLEINFORESPONSE']._serialized_start=3633
  _globals['_GETTABLEINFORESPONSE']._serialized_end=3748
  _globals['_TABLEINFO']._serialized_start=3751
  _globals['_TABLEINFO']._serialized_end=3891
  _globals['_COLUMNINFO']._serialized_start=3893
  _globals['_COLUMNINFO']._serialized_end=3989
  _globals['_LISTTABLESREQUEST']._serialized_start=3991
  _globals['_LISTTABLESREQUEST']._serialized_end=4073
  _globals['_LISTTABLESRESPONSE']._serialized_start=4075
  _globals['_LISTTABLESRESPONSE']._serialized_end=4159
  _globals['_TABLEEXISTSREQUEST']._serialized_start=4161
  _globals['_TABLEEXISTSREQUEST']._serialized_end=4259
  _globals['_TABLEEXISTSRESPONSE']._serialized_start=4261
  _globals['_TABLEEXISTSRESPONSE']._serialized_end=4346
  _globals['_HEALTHCHECKREQUEST']._serialized_start=4348
  _globals['_HEALTHCHECKREQUEST']._serialized_end=4433
  _globals['_HEALTHCHECKRESPONSE']._serialized_start=4436
  _globals['_HEALTHCHECKRESPONSE']._serialized_end=4597
  _globals['_GETSTATSREQUEST']._serialized_start=4599
  _globals['_GETSTATSREQUEST']._serialized_end=4663
  _globals['_GETSTATSRESPONSE']._serialized_start=4666
  _globals['_GETSTATSRESPONSE']._serialized_end=4834
  _globals['_CONNECTIONPOOLSTATS']._serialized_start=4837
  _globals['_CONNECTIONPOOLSTATS']._serialized_end=4986
  _globals['_DATABASESTATS']._serialized_start=4989
  _globals['_DATABASESTATS']._serialized_end=5149
  _globals['_POSTGRESSERVICE']._serialized_start=5152
  _globals['_POSTGRESSERVICE']._serialized_end=6627
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
MQTT gRPC Client
"""

from typing import List, Dict, Optional, Callable, TYPE_CHECKING
from .base_client import BaseGRPCClient
from .proto import mqtt_service_pb2, mqtt_service_pb2_grpc

if TYPE_CHECKING:
    from .consul_client import ConsulRegistry


class MQTTClient(BaseGRPCClient):
    """MQTT gRPC client"""

    def __init__(self, host: Optional[str] = None, port: Optional[int] = None, user_id: Optional[str] = None,
                 organization_id: Optional[str] = None, lazy_connect: bool = True,
                 enable_compression: bool = True, enable_retry: bool = True,
                 consul_registry: Optional['ConsulRegistry'] = None, service_name_override: Optional[str] = None):
        """
        Initialize MQTT client

        Args:
            host: Service address (optional, will use Consul discovery if not provided)
            port: Service port (optional, will use Consul discovery if not provided)
            user_id: User ID
            organization_id: Organization ID
            lazy_connect: Lazy connection (default: True)
            enable_compression: Enable compression (default: True)
            enable_retry: Enable retry (default: True)
            consul_registry: ConsulRegistry instance for service discovery (optional)
            service_name_override: Override service name for Consul lookup (optional, defaults to 'mqtt')
        """
        super().__init__(
            host=host,
            port=port,
            user_id=user_id,
            lazy_connect=lazy_connect,
            enable_compression=enable_compression,
            enable_retry=enable_retry,
            consul_registry=consul_registry,
            service_name_override=service_name_override
        )
        self.organization_id = organization_id or 'default-org'

    def _create_stub(self):
        """Create MQTT service stub"""
        return mqtt_service_pb2_grpc.MQTTServiceStub(self.channel)

    def service_name(self) -> str:
        return "MQTT"

    def default_port(self) -> int:
        return 50053

    def health_check(self, deep_check: bool = False) -> Optional[Dict]:
        """Health check"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.MQTTHealthCheckRequest(deep_check=deep_check)
            response = self.stub.HealthCheck(request)

            print(f"✅ [MQTT] Healthy: {response.healthy}")
            print(f"   Broker status: {response.broker_status}")
            print(f"   Active connections: {response.active_connections}")

            return {
                'healthy': response.healthy,
                'broker_status': response.broker_status,
                'active_connections': response.active_connections,
                'message': response.message
            }

        except Exception as e:
            return self.handle_error(e, "health check")

    def connect(self, client_id: str, username: str = '', password: str = '') -> Optional[Dict]:
        """Connect to MQTT service"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.ConnectRequest(
                client_id=client_id,
                user_id=self.user_id,
                username=username,
                password=password
            )

            response = self.stub.Connect(request)

            if response.success:
                print(f"✅ [MQTT] Connected: {client_id}")
                print(f"   Session ID: {response.session_id}")
                return {
                    'success': True,
                    'session_id': response.session_id,
                    'message': response.message
                }
            else:
                print(f"⚠️  [MQTT] Connection failed: {response.message}")
                return None

        except Exception as e:
            return self.handle_error(e, "connect")

    def disconnect(self, session_id: str) -> Optional[Dict]:
        """Disconnect"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.DisconnectRequest(
                session_id=session_id,
                user_id=self.user_id
            )

            response = self.stub.Disconnect(request)

            if response.success:
                print(f"✅ [MQTT] Disconnected")
                return {
                    'success': True,
                    'message': response.message
                }
            else:
                print(f"⚠️  [MQTT] Disconnect failed: {response.message}")
                return None

        except Exception as e:
            return self.handle_error(e, "disconnect")

    def get_connection_status(self, session_id: str) -> Optional[Dict]:
        """Get connection status"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.ConnectionStatusRequest(
                session_id=session_id,
                user_id=self.user_id
            )

            response = self.stub.GetConnectionStatus(request)

            print(f"✅ [MQTT] Connection status: {'Connected' if response.connected else 'Disconnected'}")
            print(f"   Messages sent: {response.messages_sent}")
            print(f"   Messages received: {response.messages_received}")

            return {
                'connected': response.connected,
                'connected_at': response.connected_at,
                'messages_sent': response.messages_sent,
                'messages_received': response.messages_received
            }

        except Exception as e:
            return self.handle_error(e, "get connection status")

    def publish(self, session_id: str, topic: str, payload: bytes, qos: int = 1, retained: bool = False) -> Optional[Dict]:
        """Publish message"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.PublishRequest(
                user_id=self.user_id,
                session_id=session_id,
                topic=topic,
                payload=payload,
                qos=qos,
                retained=retained
            )

            response = self.stub.Publish(request)

            if response.success:
                print(f"✅ [MQTT] Message published: {topic}")
                return {
                    'success': True,
                    'message_id': response.message_id
                }
            else:
                print(f"⚠️  [MQTT] Publish failed: {response.message}")
                return None

        except Exception as e:
            return self.handle_error(e, "publish")

    def publish_batch(self, session_id: str, messages: List[Dict]) -> Optional[Dict]:
        """
        Publish multiple messages in batch
        
        Args:
            session_id: Session ID
            messages: List of dicts with keys: topic, payload, qos, retained
        """
        try:
            self._ensure_connected()
            
            publish_requests = []
            for msg in messages:
                pub_req = mqtt_service_pb2.PublishRequest(
                    user_id=self.user_id,
                    session_id=session_id,
                    topic=msg.get('topic'),
                    payload=msg.get('payload', b''),
                    qos=msg.get('qos', 1),
                    retained=msg.get('retained', False)
                )
                publish_requests.append(pub_req)
            
            request = mqtt_service_pb2.PublishBatchRequest(
                user_id=self.user_id,
                session_id=session_id,
                messages=publish_requests
            )

            response = self.stub.PublishBatch(request)

            if response.success:
                print(f"✅ [MQTT] Batch published: {response.published_count}/{len(messages)} messages")
                return {
                    'success': True,
                    'published_count': response.published_count,
                    'failed_count': response.failed_count,
                    'message_ids': list(response.message_ids),
                    'errors': list(response.errors)
                }
            else:
                print(f"⚠️  [MQTT] Batch publish had failures: {response.failed_count}")
                return None

        except Exception as e:
            return self.handle_error(e, "publish batch")

    def publish_json(self, session_id: str, topic: str, data: Dict, qos: int = 1, retained: bool = False) -> Optional[Dict]:
        """Publish JSON message"""
        try:
            self._ensure_connected()
            
            from google.protobuf.struct_pb2 import Struct
            struct_data = Struct()
            struct_data.update(data)
            
            request = mqtt_service_pb2.PublishJSONRequest(
                user_id=self.user_id,
                session_id=session_id,
                topic=topic,
                data=struct_data,
                qos=qos,
                retained=retained
            )

            response = self.stub.PublishJSON(request)

            if response.success:
                print(f"✅ [MQTT] JSON message published: {topic}")
                return {
                    'success': True,
                    'message_id': response.message_id
                }
            else:
                print(f"⚠️  [MQTT] JSON publish failed: {response.message}")
                return None

        except Exception as e:
            return self.handle_error(e, "publish JSON")

    def subscribe(self, session_id: str, topic_filter: str, qos: int = 1, callback=None):
        """
        Subscribe to topic (streaming)
        
        Args:
            session_id: Session ID
            topic_filter: Topic filter (supports wildcards +, #)
            qos: QoS level
            callback: Function to call for each message (takes topic, payload, qos, retained, timestamp)
        """
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.SubscribeRequest(
                user_id=self.user_id,
                session_id=session_id,
                topic_filter=topic_filter,
                qos=qos
            )

            print(f"✅ [MQTT] Subscribing to: {topic_filter}")
            
            for message in self.stub.Subscribe(request):
                if callback:
                    callback(message.topic, message.payload, message.qos, 
                            message.retained, message.timestamp)
                else:
                    print(f"📩 [MQTT] Message from {message.topic}: {message.payload[:50]}")

        except Exception as e:
            self.handle_error(e, "subscribe")

    def subscribe_multiple(self, session_id: str, subscriptions: List[Dict], callback=None):
        """
        Subscribe to multiple topics
        
        Args:
            session_id: Session ID
            subscriptions: List of dicts with keys: topic_filter, qos
            callback: Function to call for each message
        """
        try:
            self._ensure_connected()
            
            topic_subs = []
            for sub in subscriptions:
                topic_sub = mqtt_service_pb2.TopicSubscription(
                    topic_filter=sub.get('topic_filter'),
                    qos=sub.get('qos', 1)
                )
                topic_subs.append(topic_sub)
            
            request = mqtt_service_pb2.SubscribeMultipleRequest(
                user_id=self.user_id,
                session_id=session_id,
                subscriptions=topic_subs
            )

            print(f"✅ [MQTT] Subscribing to {len(subscriptions)} topics")
            
            for message in self.stub.SubscribeMultiple(request):
                if callback:
                    callback(message.topic, message.payload, message.qos,
                            message.retained, message.timestamp)
                else:
                    print(f"📩 [MQTT] Message from {message.topic}: {message.payload[:50]}")

        except Exception as e:
            self.handle_error(e, "subscribe multiple")

    def unsubscribe(self, session_id: str, topic_filters: List[str]) -> Optional[int]:
        """Unsubscribe from topics"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.UnsubscribeRequest(
                user_id=self.user_id,
                session_id=session_id,
                topic_filters=topic_filters
            )

            response = self.stub.Unsubscribe(request)

            if response.success:
                print(f"✅ [MQTT] Unsubscribed from {response.unsubscribed_count} topics")
                return response.unsubscribed_count
            else:
                print(f"⚠️  [MQTT] Unsubscribe failed: {response.message}")
                return None

        except Exception as e:
            return self.handle_error(e, "unsubscribe")

    def list_subscriptions(self, session_id: str) -> List[Dict]:
        """List active subscriptions"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.ListSubscriptionsRequest(
                user_id=self.user_id,
                session_id=session_id
            )

            response = self.stub.ListSubscriptions(request)

            subscriptions = []
            for sub in response.subscriptions:
                subscriptions.append({
                    'topic_filter': sub.topic_filter,
                    'qos': sub.qos
                })
            
            print(f"✅ [MQTT] Listed {len(subscriptions)} subscriptions")
            return subscriptions

        except Exception as e:
            return self.handle_error(e, "list subscriptions") or []

    # ============================================
    # Device Management
    # ============================================

    def register_device(self, device_id: str, device_name: str, device_type: str = 'sensor',
                       metadata: Dict[str, str] = None) -> Optional[Dict]:
        """Register IoT device"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.RegisterDeviceRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                device_id=device_id,
                device_name=device_name,
                device_type=device_type,
                metadata=metadata or {}
            )

            response = self.stub.RegisterDevice(request)

            if response.success:
                print(f"✅ [MQTT] Device registered: {device_id} ({device_type})")
                return {
                    'success': True,
                    'device': response.device,
                    'message': response.message
                }
            else:
                print(f"⚠️  [MQTT] Device registration failed: {response.message}")
                return None

        except Exception as e:
            return self.handle_error(e, "register device")

    def unregister_device(self, device_id: str) -> bool:
        """Unregister device"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.UnregisterDeviceRequest(
                user_id=self.user_id,
                device_id=device_id
            )

            response = self.stub.UnregisterDevice(request)

            if response.success:
                print(f"✅ [MQTT] Device unregistered: {device_id}")
                return True
            else:
                print(f"⚠️  [MQTT] Unregister failed: {response.message}")
                return False

        except Exception as e:
            self.handle_error(e, "unregister device")
            return False

    def list_devices(self, status: int = None, page: int = 1, page_size: int = 50) -> Optional[Dict]:
        """List registered devices"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.ListDevicesRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                status=status or 0,
                page=page,
                page_size=page_size
            )

            response = self.stub.ListDevices(request)

            devices = []
            for device in response.devices:
                devices.append({
                    'device_id': device.device_id,
                    'device_name': device.device_name,
                    'device_type': device.device_type,
                    'status': device.status,
                    'registered_at': device.registered_at,
                    'last_seen': device.last_seen
                })
            
            print(f"✅ [MQTT] Listed {len(devices)} devices (total: {response.total_count})")
            return {
                'devices': devices,
                'total_count': response.total_count,
                'page': response.page,
                'page_size': response.page_size
            }

        except Exception as e:
            return self.handle_error(e, "list devices")

    def get_device_info(self, device_id: str) -> Optional[Dict]:
        """Get device information"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.GetDeviceInfoRequest(
                user_id=self.user_id,
                device_id=device_id
            )

            response = self.stub.GetDeviceInfo(request)

            device = response.device
            info = {
                'device_id': device.device_id,
                'device_name': device.device_name,
                'device_type': device.device_type,
                'status': device.status,
                'registered_at': device.registered_at,
                'last_seen': device.last_seen,
                'metadata': dict(device.metadata),
                'subscribed_topics': list(device.subscribed_topics),
                'messages_sent': device.messages_sent,
                'messages_received': device.messages_received
            }
            
            print(f"✅ [MQTT] Device info: {device_id}")
            print(f"   Type: {device.device_type}, Status: {device.status}")
            return info

        except Exception as e:
            return self.handle_error(e, "get device info")

    def update_device_status(self, device_id: str, status: int, metadata: Dict[str, str] = None) -> Optional[Dict]:
        """Update device status"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.UpdateDeviceStatusRequest(
                user_id=self.user_id,
                device_id=device_id,
                status=status,
                metadata=metadata or {}
            )

            response = self.stub.UpdateDeviceStatus(request)

            if response.success:
                print(f"✅ [MQTT] Device status updated: {device_id} -> {status}")
                return {'success': True, 'device': response.device}
            else:
                print(f"⚠️  [MQTT] Status update failed")
                return None

        except Exception as e:
            return self.handle_error(e, "update device status")

    # ============================================
    # Topic Management
    # ============================================

    def get_topic_info(self, topic: str) -> Optional[Dict]:
        """Get topic information"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.GetTopicInfoRequest(
                user_id=self.user_id,
                topic=topic
            )

            response = self.stub.GetTopicInfo(request)

            topic_info = response.topic_info
            info = {
                'topic': topic_info.topic,
                'subscriber_count': topic_info.subscriber_count,
                'message_count': topic_info.message_count,
                'last_message_time': topic_info.last_message_time,
                'has_retained_message': topic_info.has_retained_message
            }
            
            print(f"✅ [MQTT] Topic info: {topic}")
            print(f"   Subscribers: {topic_info.subscriber_count}")
            print(f"   Messages: {topic_info.message_count}")
            return info

        except Exception as e:
            return self.handle_error(e, "get topic info")

    def list_topics(self, prefix: str = '', page: int = 1, page_size: int = 50) -> Optional[Dict]:
        """List topics"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.ListTopicsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                prefix=prefix,
                page=page,
                page_size=page_size
            )

            response = self.stub.ListTopics(request)

            topics = []
            for topic_info in response.topics:
                topics.append({
                    'topic': topic_info.topic,
                    'subscriber_count': topic_info.subscriber_count,
                    'message_count': topic_info.message_count
                })
            
            print(f"✅ [MQTT] Listed {len(topics)} topics (total: {response.total_count})")
            return {
                'topics': topics,
                'total_count': response.total_count
            }

        except Exception as e:
            return self.handle_error(e, "list topics")

    def validate_topic(self, topic: str, allow_wildcards: bool = False) -> Optional[Dict]:
        """Validate topic name"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.ValidateTopicRequest(
                topic=topic,
                allow_wildcards=allow_wildcards
            )

            response = self.stub.ValidateTopic(request)

            if response.valid:
                print(f"✅ [MQTT] Topic valid: {topic}")
            else:
                print(f"⚠️  [MQTT] Topic invalid: {response.message}")

            return {
                'valid': response.valid,
                'message': response.message
            }

        except Exception as e:
            return self.handle_error(e, "validate topic")

    # ============================================
    # Retained Messages
    # ============================================

    def set_retained_message(self, topic: str, payload: bytes, qos: int = 1) -> bool:
        """Set retained message"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.SetRetainedMessageRequest(
                user_id=self.user_id,
                topic=topic,
                payload=payload,
                qos=qos
            )

            response = self.stub.SetRetainedMessage(request)

            if response.success:
                print(f"✅ [MQTT] Retained message set: {topic}")
                return True
            else:
                print(f"⚠️  [MQTT] Set retained message failed: {response.message}")
                return False

        except Exception as e:
            self.handle_error(e, "set retained message")
            return False

    def get_retained_message(self, topic: str) -> Optional[Dict]:
        """Get retained message"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.GetRetainedMessageRequest(
                user_id=self.user_id,
                topic=topic
            )

            response = self.stub.GetRetainedMessage(request)

            if response.found:
                message = response.message
                print(f"✅ [MQTT] Retained message found: {topic}")
                return {
                    'found': True,
                    'topic': message.topic,
                    'payload': message.payload,
                    'qos': message.qos,
                    'timestamp': message.timestamp
                }
            else:
                print(f"⚠️  [MQTT] No retained message on topic: {topic}")
                return {'found': False}

        except Exception as e:
            return self.handle_error(e, "get retained message")

    def delete_retained_message(self, topic: str) -> bool:
        """Delete retained message"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.DeleteRetainedMessageRequest(
                user_id=self.user_id,
                topic=topic
            )

            response = self.stub.DeleteRetainedMessage(request)

            if response.success:
                print(f"✅ [MQTT] Retained message deleted: {topic}")
                return True
            else:
                print(f"⚠️  [MQTT] Delete retained message failed: {response.message}")
                return False

        except Exception as e:
            self.handle_error(e, "delete retained message")
            return False

    # ============================================
    # Monitoring Operations
    # ============================================

    def get_statistics(self) -> Optional[Dict]:
        """Get statistics"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.GetStatisticsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id
            )

            response = self.stub.GetStatistics(request)

            stats = {
                'total_devices': response.total_devices,
                'online_devices': response.online_devices,
                'total_topics': response.total_topics,
                'total_subscriptions': response.total_subscriptions,
                'messages_sent_today': response.messages_sent_today,
                'messages_received_today': response.messages_received_today,
                'active_sessions': response.active_sessions
            }

            print(f"✅ [MQTT] Statistics:")
            print(f"   Total devices: {stats['total_devices']}")
            print(f"   Online devices: {stats['online_devices']}")
            print(f"   Total topics: {stats['total_topics']}")
            print(f"   Active sessions: {stats['active_sessions']}")

            return stats

        except Exception as e:
            return self.handle_error(e, "get statistics")

    def get_device_metrics(self, device_id: str, start_time=None, end_time=None) -> Optional[Dict]:
        """Get device metrics"""
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.GetDeviceMetricsRequest(
                user_id=self.user_id,
                device_id=device_id,
                start_time=start_time,
                end_time=end_time
            )

            response = self.stub.GetDeviceMetrics(request)

            metrics = {
                'device_id': response.device_id,
                'messages_sent': response.messages_sent,
                'messages_received': response.messages_received,
                'bytes_sent': response.bytes_sent,
                'bytes_received': response.bytes_received,
                'message_rate': [(p.timestamp, p.value) for p in response.message_rate],
                'error_rate': [(p.timestamp, p.value) for p in response.error_rate]
            }
            
            print(f"✅ [MQTT] Device metrics: {device_id}")
            print(f"   Messages: {response.messages_sent} sent, {response.messages_received} received")
            print(f"   Bytes: {response.bytes_sent} sent, {response.bytes_received} received")
            return metrics

        except Exception as e:
            return self.handle_error(e, "get device metrics")

    # ============================================
    # 设备消息监听（新增 - 替代 Gateway MQTT Adapter）
    # ============================================

    def subscribe_device_messages(self, organization_id: Optional[str] = None,
                                  message_types: List[int] = None,
                                  device_ids: List[str] = None,
                                  topic_patterns: List[str] = None,
                                  callback=None):
        """
        订阅所有设备消息流（替代 Gateway MQTT Adapter）

        这个方法会持续监听 MQTT 设备消息并通过 gRPC Stream 接收

        Args:
            organization_id: 组织 ID（可选，用于过滤）
            message_types: 订阅的消息类型列表（可选，默认所有类型）
                          1=TELEMETRY, 2=STATUS, 3=AUTH, 4=REGISTRATION, 5=COMMAND_RESPONSE, 6=NOTIFICATION_ACK
            device_ids: 订阅特定设备（可选，默认所有设备）
            topic_patterns: 自定义 topic 模式（可选，如 ['devices/+/telemetry']）
            callback: 回调函数 (device_id, message_type, topic, payload, timestamp, metadata)

        Example:
            def handle_device_message(device_id, message_type, topic, payload, timestamp, metadata):
                print(f"Device {device_id} sent {message_type}: {payload}")

            mqtt_client.subscribe_device_messages(
                message_types=[1, 2],  # 只监听 TELEMETRY 和 STATUS
                callback=handle_device_message
            )
        """
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.SubscribeDeviceMessagesRequest(
                user_id=self.user_id,
                organization_id=organization_id or '',
                message_types=message_types or [],
                device_ids=device_ids or [],
                topic_patterns=topic_patterns or []
            )

            print(f"✅ [MQTT] Subscribing to device messages...")
            if message_types:
                print(f"   Message types: {message_types}")
            if device_ids:
                print(f"   Device IDs: {device_ids}")
            if topic_patterns:
                print(f"   Topic patterns: {topic_patterns}")

            for message in self.stub.SubscribeDeviceMessages(request):
                if callback:
                    callback(
                        message.device_id,
                        message.message_type,
                        message.topic,
                        message.payload,
                        message.timestamp,
                        dict(message.metadata)
                    )
                else:
                    print(f"📩 [MQTT] Device message from {message.device_id}")
                    print(f"   Type: {message.message_type}")
                    print(f"   Topic: {message.topic}")
                    print(f"   Payload: {message.payload[:100]}")

        except Exception as e:
            self.handle_error(e, "subscribe device messages")

    # ============================================
    # Webhook 回调（新增）
    # ============================================

    def register_webhook(self, url: str,
                        organization_id: Optional[str] = None,
                        message_types: List[int] = None,
                        device_ids: List[str] = None,
                        topic_patterns: List[str] = None,
                        headers: Dict[str, str] = None,
                        secret: Optional[str] = None) -> Optional[Dict]:
        """
        注册 webhook 用于接收设备消息

        Args:
            url: 回调 URL（必须是 http/https）
            organization_id: 组织 ID
            message_types: 订阅的消息类型
            device_ids: 订阅的设备 ID（空表示所有）
            topic_patterns: Topic 模式（如 ['devices/+/telemetry']）
            headers: 自定义 HTTP Headers（如认证 token）
            secret: 签名密钥（用于验证回调）

        Returns:
            Dict with webhook_id and webhook info

        Example:
            result = mqtt_client.register_webhook(
                url="http://device-service:8201/api/v1/mqtt/webhook",
                message_types=[1, 2],  # TELEMETRY and STATUS
                headers={"Authorization": "Bearer YOUR_TOKEN"},
                secret="your-secret-key"
            )
            print(f"Webhook ID: {result['webhook_id']}")
        """
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.RegisterWebhookRequest(
                user_id=self.user_id,
                organization_id=organization_id or '',
                url=url,
                message_types=message_types or [],
                device_ids=device_ids or [],
                topic_patterns=topic_patterns or [],
                headers=headers or {},
                secret=secret or ''
            )

            response = self.stub.RegisterWebhook(request)

            if response.success:
                print(f"✅ [MQTT] Webhook registered: {response.webhook_id}")
                print(f"   URL: {url}")
                return {
                    'success': True,
                    'webhook_id': response.webhook_id,
                    'webhook': {
                        'webhook_id': response.webhook.webhook_id,
                        'url': response.webhook.url,
                        'enabled': response.webhook.enabled,
                        'success_count': response.webhook.success_count,
                        'failure_count': response.webhook.failure_count,
                    }
                }
            else:
                print(f"⚠️  [MQTT] Webhook registration failed: {response.message}")
                return None

        except Exception as e:
            return self.handle_error(e, "register webhook")

    def unregister_webhook(self, webhook_id: str) -> bool:
        """
        注销 webhook

        Args:
            webhook_id: Webhook ID

        Returns:
            True if successful
        """
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.UnregisterWebhookRequest(
                user_id=self.user_id,
                webhook_id=webhook_id
            )

            response = self.stub.UnregisterWebhook(request)

            if response.success:
                print(f"✅ [MQTT] Webhook unregistered: {webhook_id}")
                return True
            else:
                print(f"⚠️  [MQTT] Unregister failed: {response.message}")
                return False

        except Exception as e:
            self.handle_error(e, "unregister webhook")
            return False

    def list_webhooks(self, organization_id: Optional[str] = None,
                     include_disabled: bool = False) -> List[Dict]:
        """
        列出已注册的 webhooks

        Args:
            organization_id: 组织 ID（可选）
            include_disabled: 是否包含已禁用的

        Returns:
            List of webhook info dicts
        """
        try:
            self._ensure_connected()
            request = mqtt_service_pb2.ListWebhooksRequest(
                user_id=self.user_id,
                organization_id=organization_id or '',
                include_disabled=include_disabled
            )

            response = self.stub.ListWebhooks(request)

            webhooks = []
            for webhook in response.webhooks:
                webhooks.append({
                    'webhook_id': webhook.webhook_id,
                    'url': webhook.url,
                    'enabled': webhook.enabled,
                    'message_types': list(webhook.message_types),
                    'device_ids': list(webhook.device_ids),
                    'topic_patterns': list(webhook.topic_patterns),
                    'success_count': webhook.success_count,
                    'failure_count': webhook.failure_count,
                })

            print(f"✅ [MQTT] Listed {len(webhooks)} webhooks")
            return webhooks

        except Exception as e:
            return self.handle_error(e, "list webhooks") or []


# Quick test
if __name__ == '__main__':
    with MQTTClient(host='localhost', port=50053, user_id='test_user', organization_id='test_org') as client:
        # Health check
        client.health_check()

        # Connect
        conn = client.connect('test-client-001')
        
        if conn:
            session_id = conn['session_id']
            
            # Validate topic
            client.validate_topic('sensors/temperature')

            # Publish message
            client.publish(session_id, 'sensors/temperature', b'25.5', qos=1)

            # Get statistics
            client.get_statistics()

            # Disconnect
            client.disconnect(session_id)



#!/usr/bin/env python3
"""
Async DuckDB gRPC Client
High-performance async DuckDB client using grpc.aio

Performance Benefits:
- True async I/O without GIL blocking
- Concurrent analytics queries
- Non-blocking data imports/exports
- Memory-efficient connection pooling
"""

import asyncio
import logging
import re
from typing import List, Dict, Any, Optional, TYPE_CHECKING
from .async_base_client import AsyncBaseGRPCClient
from .proto import duckdb_service_pb2, duckdb_service_pb2_grpc

if TYPE_CHECKING:
    from .consul_client import ConsulRegistry

logger = logging.getLogger(__name__)


class AsyncDuckDBClient(AsyncBaseGRPCClient):
    """Async DuckDB gRPC client for high-performance analytics."""

    def __init__(
        self,
        host: Optional[str] = None,
        port: Optional[int] = None,
        user_id: Optional[str] = None,
        lazy_connect: bool = True,
        enable_compression: bool = True,
        enable_retry: bool = True,
        consul_registry: Optional['ConsulRegistry'] = None,
        service_name_override: Optional[str] = None
    ):
        """
        Initialize async DuckDB client.

        Args:
            host: Service address (optional, will use Consul discovery if not provided)
            port: Service port (optional, will use Consul discovery if not provided)
            user_id: User ID
            lazy_connect: Lazy connection (default: True)
            enable_compression: Enable compression (default: True for large data)
            enable_retry: Enable retry (default: True)
            consul_registry: ConsulRegistry instance for service discovery (optional)
            service_name_override: Override service name for Consul lookup (optional)
        """
        super().__init__(
            host=host,
            port=port,
            user_id=user_id,
            lazy_connect=lazy_connect,
            enable_compression=enable_compression,
            enable_retry=enable_retry,
            consul_registry=consul_registry,
            service_name_override=service_name_override
        )

    def _create_stub(self):
        """Create DuckDB service stub."""
        return duckdb_service_pb2_grpc.DuckDBServiceStub(self.channel)

    def service_name(self) -> str:
        return "DuckDB"

    def default_port(self) -> int:
        return 50052

    def _get_org_id(self) -> str:
        """Get organization ID."""
        return 'default_org'

    def get_table_prefix(self) -> str:
        """Get table name prefix."""
        return f"user_{self.user_id}_"

    def qualify_table_name(self, table_name: str, use_prefix: bool = True) -> str:
        """Add user prefix to table name."""
        if use_prefix and not table_name.startswith(self.get_table_prefix()):
            return f"{self.get_table_prefix()}{table_name}"
        return table_name

    def _qualify_sql_tables(self, sql: str) -> str:
        """Auto-qualify table names in SQL with user prefix."""
        prefix = self.get_table_prefix()

        if prefix in sql:
            return sql

        cte_names = set()
        cte_pattern = r'\bWITH\s+(\w+)\s+AS\s*\('
        for match in re.finditer(cte_pattern, sql, re.IGNORECASE):
            cte_names.add(match.group(1).lower())
        chained_cte_pattern = r',\s*(\w+)\s+AS\s*\('
        for match in re.finditer(chained_cte_pattern, sql, re.IGNORECASE):
            cte_names.add(match.group(1).lower())

        patterns = [
            (r'\bFROM\s+([a-zA-Z_][a-zA-Z0-9_]*)\b', rf'FROM {prefix}\1'),
            (r'\bJOIN\s+([a-zA-Z_][a-zA-Z0-9_]*)\b', rf'JOIN {prefix}\1'),
            (r'\bINTO\s+([a-zA-Z_][a-zA-Z0-9_]*)\b', rf'INTO {prefix}\1'),
            (r'\bUPDATE\s+([a-zA-Z_][a-zA-Z0-9_]*)\b', rf'UPDATE {prefix}\1'),
            (r'\bTABLE\s+([a-zA-Z_][a-zA-Z0-9_]*)\b', rf'TABLE {prefix}\1'),
        ]

        result = sql
        for pattern, replacement in patterns:
            safe_pattern = pattern.replace('([a-zA-Z_][a-zA-Z0-9_]*)',
                                          r'(?!(?:pg_|information_schema|sqlite_|duckdb_|temp\.|main\.))([a-zA-Z_][a-zA-Z0-9_]*)')

            def replace_fn(match):
                table_name = match.group(1)
                if table_name.lower() in cte_names:
                    return match.group(0)
                return match.group(0).replace(table_name, f"{prefix}{table_name}")

            result = re.sub(safe_pattern, replace_fn, result, flags=re.IGNORECASE)

        return result

    # ============================================
    # Health Check
    # ============================================

    async def health_check(self, detailed: bool = False) -> Optional[Dict]:
        """Health check."""
        try:
            await self._ensure_connected()
            request = duckdb_service_pb2.HealthCheckRequest(detailed=detailed)
            response = await self.stub.HealthCheck(request)

            return {
                'healthy': response.healthy,
                'version': getattr(response, 'version', ''),
                'message': getattr(response, 'message', '')
            }

        except Exception as e:
            return self.handle_error(e, "health check")

    # ============================================
    # Database Management
    # ============================================

    async def create_database(self, db_name: str, minio_bucket: str = '',
                             metadata: Optional[Dict[str, str]] = None) -> Optional[Dict]:
        """Create database."""
        try:
            await self._ensure_connected()
            request = duckdb_service_pb2.CreateDatabaseRequest(
                database_name=db_name,
                user_id=self.user_id,
                organization_id=self._get_org_id(),
                minio_bucket=minio_bucket,
                metadata=metadata or {},
            )

            response = await self.stub.CreateDatabase(request)

            if response.success:
                return {
                    'database_id': response.database_info.database_id,
                    'database_name': response.database_info.database_name,
                    'minio_bucket': response.database_info.minio_bucket,
                    'size_bytes': response.database_info.size_bytes,
                    'table_count': response.database_info.table_count,
                }
            return None

        except Exception as e:
            return self.handle_error(e, "create database")

    async def list_databases(self) -> List[Dict]:
        """List all databases."""
        try:
            await self._ensure_connected()
            request = duckdb_service_pb2.ListDatabasesRequest(
                user_id=self.user_id,
                organization_id=self._get_org_id(),
            )

            response = await self.stub.ListDatabases(request)

            if response.success:
                databases = []
                for db in response.databases:
                    databases.append({
                        'database_id': db.database_id,
                        'name': db.database_name,
                        'size': db.size_bytes,
                        'table_count': db.table_count,
                        'created_at': str(db.created_at),
                    })
                return databases
            return []

        except Exception as e:
            return self.handle_error(e, "list databases") or []

    async def delete_database(self, db_name: str, delete_from_minio: bool = True,
                             force: bool = False) -> bool:
        """Delete database."""
        try:
            await self._ensure_connected()
            request = duckdb_service_pb2.DeleteDatabaseRequest(
                database_id=db_name,
                user_id=self.user_id,
                delete_from_minio=delete_from_minio,
                force=force,
            )

            response = await self.stub.DeleteDatabase(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "delete database")
            return False

    async def get_database_info(self, db_name: str) -> Optional[Dict]:
        """Get database information."""
        try:
            await self._ensure_connected()
            request = duckdb_service_pb2.GetDatabaseInfoRequest(
                database_id=db_name,
                user_id=self.user_id,
            )

            response = await self.stub.GetDatabaseInfo(request)

            if response.success:
                return {
                    'database_name': response.database_info.database_name,
                    'size_bytes': response.database_info.size_bytes,
                    'table_count': response.database_info.table_count,
                    'minio_bucket': response.database_info.minio_bucket,
                    'minio_path': response.database_info.minio_path,
                    'created_at': str(response.database_info.created_at),
                    'last_accessed': str(response.database_info.last_accessed),
                }
            return None

        except Exception as e:
            return self.handle_error(e, "get database info")

    # ============================================
    # Query Operations
    # ============================================

    async def execute_query(self, db_name: str, sql: str, limit: int = 100,
                           auto_qualify_tables: bool = True) -> List[Dict]:
        """Execute SQL query."""
        try:
            await self._ensure_connected()

            if auto_qualify_tables:
                sql = self._qualify_sql_tables(sql)

            request = duckdb_service_pb2.ExecuteQueryRequest(
                database_id=db_name,
                user_id=self.user_id,
                query=sql,
                max_rows=limit,
            )

            response = await self.stub.ExecuteQuery(request)

            if response.success:
                columns = list(response.columns)
                results = []

                for row_msg in response.rows:
                    row_dict = {}
                    for i, value in enumerate(row_msg.values):
                        if i >= len(columns):
                            break
                        col_name = columns[i]

                        if value.HasField('int_value'):
                            row_dict[col_name] = value.int_value
                        elif value.HasField('double_value'):
                            row_dict[col_name] = value.double_value
                        elif value.HasField('string_value'):
                            row_dict[col_name] = value.string_value
                        elif value.HasField('bool_value'):
                            row_dict[col_name] = value.bool_value
                        elif value.HasField('null_value'):
                            row_dict[col_name] = None
                        else:
                            row_dict[col_name] = None
                    results.append(row_dict)

                return results
            return []

        except Exception as e:
            return self.handle_error(e, "execute query") or []

    async def execute_statement(self, db_name: str, sql: str,
                               auto_qualify_tables: bool = True) -> int:
        """Execute write operation (INSERT/UPDATE/DELETE)."""
        try:
            await self._ensure_connected()

            if auto_qualify_tables:
                sql = self._qualify_sql_tables(sql)

            request = duckdb_service_pb2.ExecuteStatementRequest(
                database_id=db_name,
                user_id=self.user_id,
                statement=sql,
            )

            response = await self.stub.ExecuteStatement(request)

            if response.success:
                return response.affected_rows
            return 0

        except Exception as e:
            return self.handle_error(e, "execute statement") or 0

    async def execute_batch(self, db_name: str, statements: List[str],
                           use_transaction: bool = True,
                           auto_qualify_tables: bool = True) -> Optional[Dict]:
        """Batch execute SQL statements."""
        try:
            await self._ensure_connected()

            if auto_qualify_tables:
                statements = [self._qualify_sql_tables(stmt) for stmt in statements]

            request = duckdb_service_pb2.ExecuteBatchRequest(
                database_id=db_name,
                user_id=self.user_id,
                statements=statements,
                transaction=use_transaction,
            )

            response = await self.stub.ExecuteBatch(request)

            if response.success:
                results = []
                for r in response.results:
                    results.append({
                        'success': r.success,
                        'affected_rows': r.affected_rows,
                        'error': r.error if r.error else None
                    })
                return {
                    'success': True,
                    'results': results,
                    'total_execution_time_ms': response.total_execution_time_ms
                }
            return None

        except Exception as e:
            return self.handle_error(e, "execute batch")

    # ============================================
    # Table Management
    # ============================================

    async def create_table(self, db_name: str, table_name: str,
                          schema: Dict[str, str]) -> bool:
        """Create table."""
        try:
            await self._ensure_connected()
            columns = [
                duckdb_service_pb2.ColumnInfo(name=name, data_type=dtype)
                for name, dtype in schema.items()
            ]

            request = duckdb_service_pb2.CreateTableRequest(
                database_id=db_name,
                user_id=self.user_id,
                table_name=table_name,
                columns=columns,
            )

            response = await self.stub.CreateTable(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "create table")
            return False

    async def list_tables(self, db_name: str) -> List[str]:
        """List all tables."""
        try:
            await self._ensure_connected()
            request = duckdb_service_pb2.ListTablesRequest(
                database_id=db_name,
                user_id=self.user_id,
            )

            response = await self.stub.ListTables(request)

            if response.success:
                return [table.table_name for table in response.tables]
            return []

        except Exception as e:
            return self.handle_error(e, "list tables") or []

    async def drop_table(self, db_name: str, table_name: str,
                        if_exists: bool = True, cascade: bool = False) -> bool:
        """Drop table."""
        try:
            await self._ensure_connected()
            request = duckdb_service_pb2.DropTableRequest(
                database_id=db_name,
                user_id=self.user_id,
                table_name=table_name,
                if_exists=if_exists,
                cascade=cascade,
            )

            response = await self.stub.DropTable(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "drop table")
            return False

    async def get_table_schema(self, db_name: str, table_name: str) -> Optional[Dict]:
        """Get table schema."""
        try:
            await self._ensure_connected()
            request = duckdb_service_pb2.GetTableSchemaRequest(
                database_id=db_name,
                user_id=self.user_id,
                table_name=table_name,
            )

            response = await self.stub.GetTableSchema(request)

            if response.success:
                columns = []
                for col in response.table_info.columns:
                    columns.append({
                        'name': col.name,
                        'data_type': col.data_type,
                        'nullable': col.nullable,
                        'is_primary_key': col.is_primary_key,
                    })

                return {
                    'table_name': response.table_info.table_name,
                    'columns': columns,
                    'row_count': response.table_info.row_count,
                    'size_bytes': response.table_info.size_bytes,
                }
            return None

        except Exception as e:
            return self.handle_error(e, "get table schema")

    async def get_table_stats(self, db_name: str, table_name: str,
                             include_columns: bool = True) -> Optional[Dict]:
        """Get table statistics."""
        try:
            await self._ensure_connected()
            request = duckdb_service_pb2.GetTableStatsRequest(
                database_id=db_name,
                user_id=self.user_id,
                table_name=table_name,
                include_columns=include_columns,
            )

            response = await self.stub.GetTableStats(request)

            if response.success:
                stats = {
                    'table_name': response.stats.table_name,
                    'row_count': response.stats.row_count,
                    'size_bytes': response.stats.size_bytes,
                    'column_stats': []
                }

                if include_columns:
                    for col_stat in response.stats.column_stats:
                        stats['column_stats'].append({
                            'column_name': col_stat.column_name,
                            'distinct_count': col_stat.distinct_count,
                            'null_count': col_stat.null_count,
                        })

                return stats
            return None

        except Exception as e:
            return self.handle_error(e, "get table stats")

    # ============================================
    # Data Import/Export (MinIO)
    # ============================================

    async def import_from_minio(self, db_name: str, table_name: str,
                               bucket: str, object_key: str,
                               file_format: str = 'parquet') -> bool:
        """Import data from MinIO."""
        try:
            await self._ensure_connected()
            request = duckdb_service_pb2.ImportFromMinIORequest(
                database_id=db_name,
                user_id=self.user_id,
                table_name=table_name,
                bucket_name=bucket,
                object_key=object_key,
                format=file_format,
            )

            response = await self.stub.ImportFromMinIO(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "import from MinIO")
            return False

    async def export_to_minio(self, db_name: str, query: str, bucket: str,
                             object_key: str, file_format: str = 'parquet',
                             overwrite: bool = True,
                             auto_qualify_tables: bool = True) -> Optional[Dict]:
        """Export query results to MinIO."""
        try:
            await self._ensure_connected()

            if auto_qualify_tables:
                query = self._qualify_sql_tables(query)

            request = duckdb_service_pb2.ExportToMinIORequest(
                database_id=db_name,
                user_id=self.user_id,
                query=query,
                bucket_name=bucket,
                object_key=object_key,
                format=file_format,
                overwrite=overwrite,
            )

            response = await self.stub.ExportToMinIO(request)

            if response.success:
                return {
                    'success': True,
                    'rows_exported': response.rows_exported,
                    'file_size': response.file_size,
                    'execution_time_ms': response.execution_time_ms
                }
            return None

        except Exception as e:
            return self.handle_error(e, "export to MinIO")

    async def query_minio_file(self, db_name: str, bucket: str,
                              object_key: str, file_format: str = 'parquet',
                              limit: int = 100) -> List[Dict]:
        """Query MinIO file directly without importing."""
        try:
            await self._ensure_connected()
            query = f"SELECT * FROM $FILE LIMIT {limit}"

            request = duckdb_service_pb2.QueryMinIOFileRequest(
                database_id=db_name,
                user_id=self.user_id,
                bucket_name=bucket,
                object_key=object_key,
                format=file_format,
                query=query,
            )

            response = await self.stub.QueryMinIOFile(request)

            if response.success:
                columns = list(response.columns)
                results = []

                for row_msg in response.rows:
                    row_dict = {}
                    for i, value in enumerate(row_msg.values):
                        if i >= len(columns):
                            break
                        col_name = columns[i]

                        if value.HasField('int_value'):
                            row_dict[col_name] = value.int_value
                        elif value.HasField('double_value'):
                            row_dict[col_name] = value.double_value
                        elif value.HasField('string_value'):
                            row_dict[col_name] = value.string_value
                        elif value.HasField('bool_value'):
                            row_dict[col_name] = value.bool_value
                        elif value.HasField('null_value'):
                            row_dict[col_name] = None
                        else:
                            row_dict[col_name] = None
                    results.append(row_dict)

                return results
            return []

        except Exception as e:
            return self.handle_error(e, "query MinIO file") or []

    # ============================================
    # Concurrent Operations
    # ============================================

    async def execute_queries_concurrent(self, db_name: str,
                                        queries: List[str]) -> List[List[Dict]]:
        """Execute multiple queries concurrently."""
        tasks = [self.execute_query(db_name, q) for q in queries]
        return await asyncio.gather(*tasks)

    async def execute_statements_concurrent(self, db_name: str,
                                           statements: List[str]) -> List[int]:
        """Execute multiple statements concurrently."""
        tasks = [self.execute_statement(db_name, s) for s in statements]
        return await asyncio.gather(*tasks)
